@@ -25,7 +25,7 @@ one place for ease of maintenance.
 
 import os
 
-VERSION = '1.1.1'
+VERSION = '1.2'
 
 # define the resource paths for the ui
 UI_BUILD_FILE = os.path.join(os.getcwd(), 'MovieList.glade')
@@ -41,3 +41,5 @@ CURRENT_FILE = 'current file'
 UI_SECTION = 'ui'
 WINDOW_SIZE = 'window size'
 COLUMN_WIDTHS = 'column widths'
+MEDIA_SECTION = 'media'
+MEDIA_DIR = 'directory'
